# $Id: pmv-examples.sh,v 1.3 2003/12/12 06:30:12 ianb Exp $

# This file contains usefule bourne shell functions that use pmv.
# To have these available for use, put commands to source this file
# in your startup files.
# For instance, debian users using bash would add to either
# /etc/profile or ~/.bash_profile the following (without the leading '#') :

# . /usr/share/doc/pmv/pmv-examples

# replace the pathname with the pathname to this file on your system.
# or, you could simply insert the contents of this file into your startup script.
# Any unhappy csh users out there, let me know, and I'll do a csh version.
# This file assumes pmv is in your PATH.

# Available commands:
# pmv-stdswap  - Swaps round fields in filenames delimited by " - ".
# pmv-space2_  - converts all spaces in filename to underscores.
# pmv-_2space  - Converts all underscores in filename to spaces.
# pmv-fixcaps  - Crude attempt at capitalising filenames.
# pmv-fixcase  - Much better attempt at capitalising filenames. 
#                Needs the Text::Autoformat module from CPAN.
# pmv-number   - Numbers filenames sequentially.
# pmv-deepen   - converts dirs from "artist - album" to "artist/album"
# pmv-flatten  - converts dirs from "artist/album" to "artist - album"
# To use these commands, just use: pmv-command filenames
# eg pmv-fixcase *.mp3


# This function swaps between the filename formats:
#  "Artist - Album - 01 - Track.mp3"
# and
#  "01 - Artist - Album - Track.mp3"
# If you run it a few times, it will do the same, backwards.
function pmv-stdswap
{
	pmv '$name=~s/(.*) - (.*) - (.*) - (.*)/$3 - $1 - $2 - $4/;' "$@"
}

# Converts all spaces in filenames to underscores.
function pmv-space2_
{
	pmv 's/\s/_/g;' "$@"
}

# Converts all underscores in filenames to spaces.
function pmv-_2space
{
	pmv 's/_/ /g;' "$@"
}

# Number all supplied files increasing sequentially from 01
# The number is prepended to the filename, so
# "foo.mp3" becomes "01 - foo.mp3"
# The filename is otherwise untouched.
# This is often a useful starting point for completely disorganised filenames,
# before using freedbtofilename, etc.
function pmv-number
{
	pmv 'unless(defined($c)) { $c="00"; } $c++;$name=~s/^/$c - /;' "$@"
}

# Crude attempt to capitalise filenames.
# Doesn't handle brackets and other things very well.
function pmv-fixcaps
{
	pmv  '$name=join(" ",map({ucfirst(lc($_));} split(/\s+/,$name)));' "$@"
}

# Much better attempt to capitalise filenames
# This uses Damian Conway's incredibly smart Text::Autoformat module from CPAN
# (http://www.cpan.org/). It uses "highlight" case, ie  all "important" words 
# are capitalised. If you want to change the behaviour, find the section
# that says {case=>"highlight"} and change it as follows:
# {case=>"title"}     - Capitalise every word.
# {case=>"sentence"}  - Capitalise the first word.
# {case=>"upper"}     - Upper-case everything.
# {case=>"lower"}     - Lower-case everything. 
function pmv-fixcase
{
	pmv -b 'use Text::Autoformat;' '$name=~/(^[^\.]+\.?$)|(?:(.*)(\..+))/; $f=(defined($2))?$2:$1;$ext=$3 ;@a=split(/ - /,$f);$name=join(" - ",map {autoformat($_,{case=>"highlight"});} @a);$name=~s/\n//g;if(defined($ext)){$name.="$ext";}' "$@"
}

# Convert directories of the form "artist/album" to "artist - album"
# Note use of -b to include a module and -e to delete directories after moving
function pmv-flatten
{
    pmv -b 'use Cwd;' -e 'for my $del (@del){rmdir $del;}' '$d=$_;unless(-d $d) {die("$d: not a directory\n");}$d=Cwd::abs_path($d);($top,$mid,$last)=($d=~/(.+)\/(.+)\/(.+)/) ;push(@del,$mid);$_="$top/$mid - $last";' "$@"
}

# Convert directories of the form "artist - album" to "artist/album" 
function pmv-deepen
{
    pmv 'if(-d && /(.*) - (.*)/) { mkdir $1; $_="$1/$2";}' "$@"
}

# prepends a datestamp in the form YYYYMMDD (eg 20031211)
# (which will sort chronologically)			
function pmv-datestamp
{
	pmv -b '$d=`date +%Y%d%m`;chomp($d);' '$name = "$d.$name";' "$@"
}
